from flask import Flask, render_template, request
import pandas as pd
import numpy as np
from sklearn.metrics.pairwise import cosine_similarity
from sklearn.feature_extraction.text import TfidfVectorizer

#initialising an instance of the class Flask.
app = Flask(__name__)
#Customising display options to remove the maximum column width.
pd.set_option('display.max_colwidth', None)
#Reading the top [:number] rows of the tsv dataset and assigning names to the columns.
df = pd.read_csv('abstracts.tsv', sep='\t', names=['abstract_id', 'paper_abstract'])[:1000]
#Defining Tfidf variable to use TfidfVectorizer function from sklearn. Vectorizes the data in the dataframe.
Tfidf = TfidfVectorizer()
#Calling Sklearn's fit_transform function for second column of the data frame.
Tfidf_matrix = Tfidf.fit_transform(df['paper_abstract'])
#Calculates the cosine similarity for the 
cosine_sim = cosine_similarity(Tfidf_matrix)
#list which will be used to match the indices.
indices = pd.Series(df.index)


#function to find and return the abstract for the provided abstract id
def returned_abstract(abstract_id, df1):

	absid_returned = []
	#this finds the exact row for the abstract id and then drops the abstract id.
	absid_returned = df1.loc[df1['abstract_id'] == abstract_id].drop(columns = ['abstract_id']).iloc[0,0]
	#returning the paper abstract
	return absid_returned


#function to find the recommendations and relevancy scores.
def recommended_papers(abstract_id, cosine_sim, df):
	recommendations = []
	# gets you the position of the input abstract id

	idx = list(df.abstract_id).index(abstract_id)

	#Creates a Series with similarity scores in descending order.
	score_series = pd.Series(cosine_sim[idx]).sort_values(ascending = False)
	# this gets the list of positions of papers with highest similarity
	top10_sim = list(score_series.iloc[1:11].index)
	# but this gets the actual similarity scores (from 0, very dissimilar, to 1, same abstract)
	top10_scores = list(score_series.iloc[1:11].values)
	#populating the list
	for i in top10_sim:
		recommendations.append(df.paper_abstract.iloc[i])
	return recommendations,top10_scores


#route decorator to tell flask what URL to use
@app.route('/')
@app.route('/home')
def home():
	#to render the home html page.
    return render_template('home.html')	


#route decorator to tell flask what URL to use
@app.route('/result', methods=['POST'])
#function to call returned_abstract and recommended_papers using user input value.
def result():
	#To request the data in the form which is entered by the user.
	absid = request.form['absid']
	#Creating an empty array.
	#abc=[]
	#assigning values returned by the NLP code to variables.
	abc,scores=recommended_papers(absid, cosine_sim, df)
	#Joining the variables to form an array of tuples.
	joint = zip(abc,scores)
	#Creating an empty array.
	papername=[]
	#Assigning abstract returned by the function returned_abstract in response to the user input.
	papername=returned_abstract(absid, df)
	
	#to render the html page with results of the query with values passed as ab=joint and absid=papername.
	return render_template('returned_papers.html', ab=joint, absid=papername)


if __name__ == '__main__':
    app.run(debug=True)
    app.config['TEMPLATES_AUTO_RELOAD'] = True



